@@ -1,7 +1,7 @@
 /**
  * Provides system for code reuse via traits
  *
- *  Copyright (C) 2014, 2015 Free Software Foundation, Inc.
+ *  Copyright (C) 2014, 2015, 2016, 2017 Free Software Foundation, Inc.
  *
  *  This file is part of GNU ease.js.
  *
@@ -62,12 +62,13 @@ function Trait()
 /**
  * Create a named trait
  *
- * @param  {string}  name  trait name
- * @param  {Object}  def   trait definition
+ * @param  {string}   name  trait name
+ * @param  {Object}   dfn   trait definition
+ * @param  {Object=}  base  extension base
  *
  * @return  {Function}  named trait
  */
-function createNamedTrait( name, dfn )
+function createNamedTrait( name, dfn, base )
 {
     if ( typeof name !== 'string' )
     {
@@ -77,16 +78,26 @@ function createNamedTrait( name, dfn )
     }
 
     dfn.__name = name;
-    return Trait.extend( dfn );
+
+    var args = [ dfn ];
+    if ( base !== undefined )
+    {
+        args.unshift( base );
+    }
+
+    return Trait.extend.apply( null, args );
 }
 
 
 function _createStaging( name )
 {
     return {
-        extend: function( dfn )
+        extend: function( /*...*/ )
         {
-            return createNamedTrait( name, dfn );
+            // [base, ]dfn
+            return ( arguments.length === 1 )
+                ? createNamedTrait( name, arguments[ 0 ] )
+                : createNamedTrait( name, arguments[ 1 ], arguments[ 0 ] );
         },
 
         implement: function()
@@ -251,7 +262,7 @@ Trait.extend = function( /* ... */ )
  * into something of type S; a `TypeError` will be thrown if this contract
  * is violated.
  *
- * @param  {Class}  base  mixor (target of mixin)
+ * @param  {Class}  base  mixer (target of mixin)
  * @param  {Trait}  T     mixee (trait being mixed in)
  *
  * @return  {undefined}
@@ -272,7 +283,7 @@ function _validateMixin( base, T )
     {
         throw TypeError(
             "Cannot mix trait " + T.toString() + " into " + base.toString() +
-                "; mixor must be of type " + T.__extbase.toString()
+                "; mixer must be of type " + T.__extbase.toString()
         );
     }
 }
@@ -487,14 +498,12 @@ function createConcrete( acls )
         // parent ACLS has access to it (!), which is not prohibited since
         // JS does not provide a strict typing mechanism...this is a kluge)
         // and target supertype---that is, what __super calls should
-        // referene
-        'protected ___$$pmo$$': null,
-        'protected ___$$super$$': null,
-        __construct: function( base, pmo )
-        {
-            this.___$$super$$ = base;
-            this.___$$pmo$$   = pmo;
-        },
+        // reference
+        'protected ___$$pmo$$':     null,
+        'protected ___$$super$$':   null,
+
+        'weak virtual __construct': _fvoid,
+        'override __construct':     __tconstruct,
 
         // mainly for debugging; should really never see this.
         __name: '#ConcreteTrait#',
@@ -533,6 +542,30 @@ function createConcrete( acls )
 }
 
 
+/**
+ * Trait class __construct method
+ *
+ * This is to be used for the `__construct' method of a concrete trait
+ * class.  This simply performs low-level trait initialization.
+ *
+ * Just below this function's definition, its `__length' property is set to
+ * Infinity---this avoids supertype compatibility issues based on argument
+ * count (see argument length check in `MemberBuilderValidator').
+ *
+ * @param  {Class}   base  supertype
+ * @param  {Object}  pmo   protected member object
+ */
+function __tconstruct( base, pmo )
+{
+    this.___$$super$$ = base;
+    this.___$$pmo$$   = pmo;
+}
+
+// prevent any issues related to supertype compatability (__length is
+// recognized by easejs to store argument count)
+__tconstruct.__length = Infinity;
+
+
 /**
  * Create virtual method proxies for all virtual members
  *
@@ -556,6 +589,14 @@ function createVirtProxy( acls, dfn )
     // f = `field'
     for ( var f in vmembers )
     {
+        // constructors may exist when extending a class; they require
+        // special treatment and it makes no sense to create a proxy for
+        // them
+        if ( f === '__construct' )
+        {
+            continue;
+        }
+
         var vis = ( acls.___$$methods$$['public'][ f ] !== undefined )
             ? 'public'
             : 'protected';
@@ -906,7 +947,20 @@ function tctor( tc, base, privsym )
         // (but not private); in return, we will use its own protected
         // visibility object to gain access to its protected members...quite
         // the intimate relationship
-        this[ f ] = C( base, this[ privsym ].vis )[ privsym ].vis;
+        var tinst = C( base, this[ privsym ].vis );
+        this[ f ] = tinst[ privsym ].vis;
+
+        // TODO: this should use util.defineSecureProp
+        // we set __inst on the trait object itself (public visibility
+        // object, which is okay, since the trait is encapsulated anyway)
+        // because otherwise, in the case of an ES3 fallback, `vis' will be
+        // affected by __inst defined on `base', leading to a fight over
+        // ownership that I spent too much time trying to figure out
+        tinst.__inst = this[ privsym ].inst;
+
+        // rebind trait's supertype context (if any) to our own, causing us
+        // to share state
+        bindSuperCtx( this[ f ], this, privsym );
 
         // this has been previously validated to ensure that it is a
         // function
@@ -920,6 +974,34 @@ function tctor( tc, base, privsym )
 };
 
 
+/**
+ * Bind the supertype context to that of another instance
+ *
+ * The source instance FROM is expected to have a `___$$super$$' property
+ * holding a reference to the supertype.  The context associated with the
+ * supertype's class id X in FROM will then be reassigned by reference to
+ * the context X of instance TO.
+ *
+ * This has the effect of ensuring that, whenever a call enters the
+ * supertype's context in FROM, it will use all state from the context of
+ * TO.  For mixins, this means that---despite the trait itself inheriting
+ * from the class---we will use the private state of the _mixer_, not our
+ * own.
+ *
+ * This is all done without any loss in performance, since calling context
+ * is immediately bound rather than resolved during each method call.
+ *
+ * TODO: make into a standard API for class manipulation
+ */
+function bindSuperCtx( from, to, privsym )
+{
+    var ctx = from[ privsym ].vis,
+        cid = ctx.___$$super$$.__cid;
+
+    ctx[ privsym ].vis[ cid ] = to[ privsym ].vis[ cid ];
+}
+
+
 /**
  * Create trait constructor
  *
