@@ -1,7 +1,8 @@
 /**
  * Tests member builder validation rules
  *
- *  Copyright (C) 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
+ *  Copyright (C) 2011, 2012, 2013, 2014, 2017
+ *                Free Software Foundation, Inc.
  *
  *  This file is part of GNU ease.js.
  *
@@ -192,9 +193,18 @@ require( 'common' ).testCase(
 
 
     /**
-     * Overriding a method in ease.js does not immediately make it virtual.
-     * Rather, the virtual keyword must be explicitly specified. Let's ensure
-     * that it is permitted.
+     * An override is implicitly virtual.  This was not always the case:
+     * prior to v0.2.9, an override did not imply virtual.
+     */
+    'Can override overridden method with concrete method': function()
+    {
+        this.quickKeywordMethodTest( [ 'override' ], null, [ 'override' ] );
+    },
+
+
+    /**
+     * For legacy reasons; see above test.  In the future, this might
+     * trigger a notice, since it's redundant.  No harm in being explicit!
      */
     'Can declare override as virtual': function()
     {
